Class: Legate::CLI::Main
- Inherits:
-
BaseCommand
- Object
- Thor
- BaseCommand
- Legate::CLI::Main
- Defined in:
- lib/legate/cli.rb
Overview
Main CLI class that provides the entry point for all Legate commands
Main CLI class that provides the entry point for all Legate commands