Exception: Legate::DefinitionStore::NotFoundError
- Defined in:
- lib/legate/definition_store.rb
Overview
Error raised when a definition is not found Inherits from Legate::DefinitionStore::Error defined in errors.rb
Error raised when a definition is not found Inherits from Legate::DefinitionStore::Error defined in errors.rb