9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
681
682
683
684
685
686
687
688
689
690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
751
752
753
754
755
756
757
758
759
760
761
762
763
764
765
766
767
768
769
770
771
772
773
774
775
776
777
778
779
780
781
782
783
784
785
786
787
788
789
790
791
792
793
794
795
796
797
798
799
800
801
802
803
804
805
806
807
808
809
810
811
812
813
814
815
816
817
818
819
820
821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
846
847
848
849
850
851
852
853
854
855
856
857
858
859
860
861
862
863
864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
923
924
925
926
927
928
929
930
931
932
933
934
935
936
937
938
939
940
941
942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
964
965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
983
984
985
986
987
988
989
990
991
992
993
994
995
996
997
998
999
1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
1028
1029
1030
1031
1032
1033
1034
1035
1036
1037
1038
1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
1049
1050
1051
1052
1053
1054
1055
1056
1057
1058
1059
1060
1061
1062
1063
1064
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
1075
1076
1077
1078
1079
1080
1081
1082
1083
1084
1085
1086
1087
1088
1089
1090
1091
1092
1093
1094
1095
1096
1097
1098
1099
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
1111
1112
1113
1114
1115
1116
1117
1118
1119
1120
1121
1122
1123
1124
1125
1126
1127
1128
1129
1130
1131
1132
1133
1134
1135
1136
1137
1138
1139
1140
1141
1142
1143
1144
1145
1146
1147
1148
1149
1150
1151
1152
1153
1154
1155
1156
1157
1158
1159
1160
1161
1162
1163
1164
1165
1166
1167
1168
1169
1170
1171
1172
1173
1174
1175
1176
1177
1178
1179
1180
1181
1182
1183
1184
1185
1186
1187
1188
1189
1190
1191
1192
1193
1194
1195
1196
1197
1198
1199
1200
1201
1202
1203
1204
1205
1206
1207
1208
1209
1210
1211
1212
1213
1214
1215
1216
1217
1218
1219
1220
1221
1222
1223
1224
1225
1226
1227
1228
1229
1230
1231
1232
1233
1234
1235
1236
1237
1238
1239
1240
1241
1242
1243
1244
1245
1246
1247
1248
1249
1250
1251
1252
1253
1254
1255
1256
1257
1258
1259
1260
1261
1262
1263
1264
1265
1266
1267
1268
1269
1270
1271
1272
1273
1274
1275
1276
1277
1278
1279
1280
1281
1282
1283
1284
1285
1286
1287
1288
1289
1290
1291
1292
1293
1294
1295
1296
1297
1298
1299
1300
1301
1302
1303
1304
1305
1306
1307
1308
1309
1310
1311
1312
1313
1314
1315
1316
1317
1318
1319
1320
1321
1322
1323
1324
1325
1326
1327
1328
1329
1330
1331
1332
1333
1334
1335
1336
1337
1338
1339
1340
1341
1342
1343
1344
1345
1346
1347
1348
1349
1350
1351
1352
1353
1354
1355
1356
1357
1358
1359
1360
1361
1362
1363
1364
1365
1366
1367
1368
1369
1370
1371
1372
1373
1374
1375
1376
1377
1378
1379
1380
1381
1382
1383
1384
1385
1386
1387
1388
1389
1390
1391
1392
1393
1394
1395
1396
1397
1398
1399
1400
1401
1402
1403
1404
1405
1406
1407
1408
1409
1410
1411
1412
1413
1414
1415
1416
1417
1418
1419
1420
1421
1422
1423
1424
1425
1426
1427
1428
1429
1430
1431
1432
1433
1434
1435
1436
1437
1438
1439
1440
1441
1442
1443
1444
1445
1446
1447
1448
1449
1450
1451
1452
1453
1454
1455
1456
1457
1458
1459
1460
1461
1462
1463
1464
1465
1466
1467
1468
1469
1470
1471
1472
1473
1474
1475
1476
1477
1478
1479
1480
1481
1482
1483
1484
1485
1486
1487
1488
1489
1490
1491
1492
1493
1494
1495
1496
1497
1498
1499
1500
1501
1502
1503
1504
1505
1506
1507
1508
1509
1510
1511
1512
1513
1514
1515
1516
1517
1518
1519
1520
1521
1522
1523
1524
1525
1526
1527
1528
1529
1530
1531
1532
1533
1534
1535
1536
1537
1538
|
# File 'lib/legate/web/routes/authentication_routes.rb', line 9
def self.registered(app)
app.helpers do
def get_scheme_description(scheme)
case scheme.scheme_type
when :api_key
'Simple API key authentication for services that use API keys in headers or query parameters'
when :http_bearer
'HTTP Bearer token authentication and basic auth support'
when :oauth2
'OAuth2 authorization code flow for secure third-party authentication'
when :oidc, :openid_connect
'OpenID Connect authentication extending OAuth2 with identity information'
when :service_account
'Service account authentication with automatic token exchange'
when :google_service_account
'Google Cloud service account authentication with JSON key files'
else
"Authentication scheme of type: #{scheme.scheme_type}"
end
end
def get_credential_description(credential)
case credential.auth_type
when :api_key
'API key credential for service authentication'
when :oauth2, :oidc
'OAuth2/OIDC client credentials for authorization flows'
when :service_account, :google_service_account
'Service account credentials for automated authentication'
when :http_bearer
'Bearer token or basic auth credentials'
else
"Credential of type: #{credential.auth_type}"
end
end
def get_masked_credential_info(credential)
info_parts = []
if credential[:api_key, resolve_env: false]
masked_key = mask_sensitive_value(credential[:api_key, resolve_env: false])
info_parts << "API Key: #{masked_key}"
end
info_parts << "Client ID: #{credential[:client_id, resolve_env: false]}" if credential[:client_id, resolve_env: false]
if credential[:client_secret, resolve_env: false]
masked_secret = mask_sensitive_value(credential[:client_secret, resolve_env: false])
info_parts << "Client Secret: #{masked_secret}"
end
if credential[:bearer_token, resolve_env: false]
masked_token = mask_sensitive_value(credential[:bearer_token, resolve_env: false])
info_parts << "Bearer Token: #{masked_token}"
end
info_parts << "Username: #{credential[:username, resolve_env: false]}" if credential[:username, resolve_env: false]
info_parts << "Password: #{mask_sensitive_value(credential[:password, resolve_env: false])}" if credential[:password, resolve_env: false]
info_parts.empty? ? 'No displayable information' : info_parts.join(', ')
end
def mask_sensitive_value(value)
return '[Not Set]' if value.nil? || value.empty?
value_str = value.to_s
return '••••••••' if value_str.length < 12
"••••••••#{value_str[-4..]}"
end
def get_scheme_config_fields(scheme_type)
case scheme_type.to_sym
when :api_key
[] when :http_bearer
[] when :oauth2
[
{ name: 'authorization_url', type: 'url', required: true, label: 'Authorization URL' },
{ name: 'token_url', type: 'url', required: true, label: 'Token URL' },
{ name: 'scopes', type: 'text', required: false, label: 'Scopes (space-separated)' },
{ name: 'use_pkce', type: 'checkbox', required: false, label: 'Use PKCE' },
{ name: 'revocation_url', type: 'url', required: false, label: 'Revocation URL' }
]
when :oidc, :openid_connect
[
{ name: 'authorization_url', type: 'url', required: true, label: 'Authorization URL' },
{ name: 'token_url', type: 'url', required: true, label: 'Token URL' },
{ name: 'userinfo_url', type: 'url', required: false, label: 'UserInfo URL' },
{ name: 'scopes', type: 'text', required: false, label: 'Scopes (space-separated)' },
{ name: 'use_pkce', type: 'checkbox', required: false, label: 'Use PKCE' }
]
when :service_account
[
{ name: 'token_url', type: 'url', required: true, label: 'Token URL' },
{ name: 'scopes', type: 'text', required: false, label: 'Scopes (space-separated)' }
]
when :google_service_account
[
{ name: 'scopes', type: 'text', required: false, label: 'Scopes (space-separated)' }
]
else
[]
end
end
def get_scheme_current_config(scheme)
config = {}
case scheme.scheme_type
when :oauth2, :oidc, :openid_connect
config['authorization_url'] = scheme.authorization_url if scheme.respond_to?(:authorization_url)
config['token_url'] = scheme.token_url if scheme.respond_to?(:token_url)
config['scopes'] = scheme.scopes.join(' ') if scheme.respond_to?(:scopes) && scheme.scopes
config['use_pkce'] = scheme.use_pkce if scheme.respond_to?(:use_pkce)
config['revocation_url'] = scheme.revocation_url if scheme.respond_to?(:revocation_url)
config['userinfo_url'] = scheme.userinfo_url if scheme.respond_to?(:userinfo_url)
when :service_account, :google_service_account
config['token_url'] = scheme.token_url if scheme.respond_to?(:token_url)
config['scopes'] = scheme.scopes.join(' ') if scheme.respond_to?(:scopes) && scheme.scopes
end
config
end
def get_compatible_credential_types(scheme_type)
case scheme_type.to_sym
when :api_key
['api_key']
when :http_bearer
%w[http_bearer bearer_token]
when :oauth2, :oidc, :openid_connect
%w[oauth2 oidc]
when :service_account, :google_service_account
%w[service_account google_service_account]
else
[]
end
end
def get_credential_config_fields(auth_type)
case auth_type.to_sym
when :api_key
[
{ name: 'api_key', type: 'password', required: true, label: 'API Key', placeholder: 'Enter your API key' },
{ name: 'location', type: 'select', required: false, label: 'Location', options: %w[header query cookie], default: 'header' },
{ name: 'name', type: 'text', required: false, label: 'Parameter Name', placeholder: 'X-API-Key' }
]
when :http_bearer
[
{ name: 'bearer_token', type: 'password', required: true, label: 'Bearer Token', placeholder: 'Enter bearer token' }
]
when :oauth2, :oidc
[
{ name: 'client_id', type: 'text', required: true, label: 'Client ID', placeholder: 'Enter client ID' },
{ name: 'client_secret', type: 'password', required: true, label: 'Client Secret', placeholder: 'Enter client secret' },
{ name: 'redirect_uri', type: 'url', required: false, label: 'Redirect URI', placeholder: 'https://your-app.com/callback' },
{ name: 'scopes', type: 'text', required: false, label: 'Scopes (space-separated)', placeholder: 'read write' }
]
when :service_account
[
{ name: 'client_email', type: 'email', required: true, label: 'Client Email', placeholder: 'service-account@project.iam.gserviceaccount.com' },
{ name: 'private_key', type: 'textarea', required: true, label: 'Private Key', placeholder: '-----BEGIN PRIVATE KEY-----\n...\n-----END PRIVATE KEY-----' },
{ name: 'project_id', type: 'text', required: false, label: 'Project ID', placeholder: 'your-project-id' }
]
when :google_service_account
[
{ name: 'service_account_key', type: 'textarea', required: true, label: 'Service Account Key (JSON)', placeholder: '{"type": "service_account", "project_id": "..."}' }
]
when :basic
[
{ name: 'username', type: 'text', required: true, label: 'Username', placeholder: 'Enter username' },
{ name: 'password', type: 'password', required: true, label: 'Password', placeholder: 'Enter password' }
]
else
[]
end
end
def get_credential_current_config(credential)
config = {}
credential.to_h(resolve_env: false).each do |key, value|
next if key == :auth_type
config[key.to_s] = if sensitive_credential_field?(key)
mask_sensitive_value(value)
else
value
end
end
config
end
def sensitive_credential_field?(field_name)
sensitive_fields = %i[api_key client_secret bearer_token password private_key service_account_key token]
sensitive_fields.include?(field_name.to_sym)
end
def get_compatible_schemes_for_credential(auth_type)
auth_manager = Legate::Auth::Manager.instance
schemes = auth_manager.schemes || {}
compatible_schemes = schemes.select do |scheme_name, scheme|
case auth_type.to_sym
when :api_key
scheme.scheme_type == :api_key
when :http_bearer
scheme.scheme_type == :http_bearer
when :oauth2, :oidc
%i[oauth2 oidc openid_connect].include?(scheme.scheme_type)
when :service_account, :google_service_account
%i[service_account google_service_account].include?(scheme.scheme_type)
when :basic
scheme.scheme_type == :http_bearer
else
false
end
end
compatible_schemes.keys
end
def get_mapping_config_fields
[
{ name: 'pattern', type: 'text', required: true, label: 'URL Pattern', placeholder: 'e.g. https://api.example.com/v1/* or ^https://.*\\.example\\.com/.*$' },
{ name: 'pattern_type', type: 'select', required: true, label: 'Pattern Type', options: %w[string regex], default: 'string' },
{ name: 'scheme_name', type: 'select', required: true, label: 'Authentication Scheme' },
{ name: 'credential_name', type: 'select', required: true, label: 'Credential' },
{ name: 'priority', type: 'number', required: false, label: 'Priority', placeholder: '1' },
{ name: 'active', type: 'checkbox', required: false, label: 'Active', default: true }
]
end
def valid_mapping_pattern?(pattern, pattern_type)
return false if pattern.nil? || pattern.strip.empty?
if pattern_type == 'regex'
begin
Regexp.new(pattern)
true
rescue RegexpError
false
end
else
true
end
end
def mapping_scheme_credential_compatible?(scheme, credential)
return false unless scheme && credential
compatible_types = get_compatible_credential_types(scheme.scheme_type)
compatible_types.include?(credential.auth_type.to_s) || compatible_types.include?(credential.auth_type)
end
def test_credential_functionality(credential, test_options = {})
test_results = { success: true, tests: [], credential_name: credential.to_s }
begin
credential.to_h(resolve_env: true)
test_results[:tests] << {
name: 'Basic Validation',
status: 'passed',
message: 'Credential structure and environment variables are valid'
}
rescue StandardError => e
test_results[:success] = false
test_results[:tests] << {
name: 'Basic Validation',
status: 'failed',
message: "Validation failed: #{e.message}"
}
return test_results
end
test_results[:tests] << case credential.auth_type
when :api_key
test_api_key_credential(credential, test_options)
when :oauth2, :oidc
test_oauth_credential(credential, test_options)
when :service_account, :google_service_account
test_service_account_credential(credential, test_options)
when :http_bearer
test_bearer_credential(credential, test_options)
else
{
name: 'Type-Specific Test',
status: 'skipped',
message: "No specific test available for credential type: #{credential.auth_type}"
}
end
test_results[:success] = test_results[:tests].all? { |test| test[:status] != 'failed' }
test_results
end
def test_api_key_credential(credential, options = {})
api_key = credential[:api_key]
return { name: 'API Key Test', status: 'failed', message: 'API key is missing' } unless api_key
if options[:test_url]
begin
require 'net/http'
Legate::Auth::UrlGuard.validate!(options[:test_url], label: 'Test URL')
uri = URI(options[:test_url])
http = Net::HTTP.new(uri.host, uri.port)
http.use_ssl = uri.scheme == 'https'
http.read_timeout = 10
request = Net::HTTP::Get.new(uri)
location = credential[:location] || 'header'
key_name = credential[:name] || 'X-API-Key'
case location
when 'header'
request[key_name] = api_key
when 'query'
uri.query = "#{key_name}=#{api_key}"
request = Net::HTTP::Get.new(uri)
end
response = http.request(request)
if response.code.to_i < 400
{ name: 'API Key Test', status: 'passed', message: "API key accepted (HTTP #{response.code})" }
else
{ name: 'API Key Test', status: 'failed', message: "API key rejected (HTTP #{response.code})" }
end
rescue StandardError => e
{ name: 'API Key Test', status: 'failed', message: "Network error: #{e.message}" }
end
else
{ name: 'API Key Test', status: 'passed', message: 'API key format is valid (no test URL provided)' }
end
end
def test_oauth_credential(credential, _options = {})
client_id = credential[:client_id]
client_secret = credential[:client_secret]
return { name: 'OAuth Test', status: 'failed', message: 'Client ID is missing' } unless client_id
return { name: 'OAuth Test', status: 'failed', message: 'Client secret is missing' } unless client_secret
{ name: 'OAuth Test', status: 'passed', message: 'OAuth credentials are properly formatted' }
end
def test_service_account_credential(credential, _options = {})
if credential.auth_type == :google_service_account
key_data = credential[:service_account_key]
return { name: 'Service Account Test', status: 'failed', message: 'Service account key is missing' } unless key_data
begin
parsed_key = JSON.parse(key_data)
required_fields = %w[type project_id private_key_id private_key client_email client_id]
missing_fields = required_fields.reject { |field| parsed_key.key?(field) }
if missing_fields.empty?
{ name: 'Service Account Test', status: 'passed', message: 'Service account key is valid JSON with all required fields' }
else
{ name: 'Service Account Test', status: 'failed', message: "Missing required fields: #{missing_fields.join(', ')}" }
end
rescue JSON::ParserError
{ name: 'Service Account Test', status: 'failed', message: 'Service account key is not valid JSON' }
end
else
client_email = credential[:client_email]
private_key = credential[:private_key]
return { name: 'Service Account Test', status: 'failed', message: 'Client email is missing' } unless client_email
return { name: 'Service Account Test', status: 'failed', message: 'Private key is missing' } unless private_key
if private_key.include?('BEGIN PRIVATE KEY')
{ name: 'Service Account Test', status: 'passed', message: 'Service account credentials are properly formatted' }
else
{ name: 'Service Account Test', status: 'failed', message: 'Private key does not appear to be in PEM format' }
end
end
end
def test_bearer_credential(credential, options = {})
bearer_token = credential[:bearer_token]
return { name: 'Bearer Token Test', status: 'failed', message: 'Bearer token is missing' } unless bearer_token
if options[:test_url]
begin
require 'net/http'
Legate::Auth::UrlGuard.validate!(options[:test_url], label: 'Test URL')
uri = URI(options[:test_url])
http = Net::HTTP.new(uri.host, uri.port)
http.use_ssl = uri.scheme == 'https'
http.read_timeout = 10
request = Net::HTTP::Get.new(uri)
request['Authorization'] = "Bearer #{bearer_token}"
response = http.request(request)
if response.code.to_i < 400
{ name: 'Bearer Token Test', status: 'passed', message: "Bearer token accepted (HTTP #{response.code})" }
else
{ name: 'Bearer Token Test', status: 'failed', message: "Bearer token rejected (HTTP #{response.code})" }
end
rescue StandardError => e
{ name: 'Bearer Token Test', status: 'failed', message: "Network error: #{e.message}" }
end
else
{ name: 'Bearer Token Test', status: 'passed', message: 'Bearer token format is valid (no test URL provided)' }
end
end
def test_authenticated_api_call(url, scheme_name, credential_name, _options = {})
auth_manager = Legate::Auth::Manager.instance
scheme = auth_manager.get_scheme(scheme_name.to_sym)
credential = auth_manager.get_credential(credential_name.to_sym)
return { success: false, error: 'Scheme not found' } unless scheme
return { success: false, error: 'Credential not found' } unless credential
return { success: false, error: 'Scheme and credential are not compatible' } unless mapping_scheme_credential_compatible?(scheme, credential)
begin
require 'net/http'
Legate::Auth::UrlGuard.validate!(url, label: 'Test URL')
uri = URI(url)
http = Net::HTTP.new(uri.host, uri.port)
http.use_ssl = uri.scheme == 'https'
http.read_timeout = 15
request = Net::HTTP::Get.new(uri)
case scheme.scheme_type
when :api_key
location = credential[:location] || 'header'
key_name = credential[:name] || 'X-API-Key'
api_key = credential[:api_key]
case location
when 'header'
request[key_name] = api_key
when 'query'
uri.query = "#{key_name}=#{api_key}"
request = Net::HTTP::Get.new(uri)
end
when :http_bearer
request['Authorization'] = "Bearer #{credential[:bearer_token]}"
when :oauth2, :oidc
return { success: false, error: 'OAuth2 testing requires a complete authentication flow' }
when :service_account, :google_service_account
return { success: false, error: 'Service account testing requires token generation' }
end
response = http.request(request)
{
success: true,
status_code: response.code.to_i,
status_message: response.message,
headers: response.to_hash,
body_preview: response.body&.slice(0, 500),
authenticated: response.code.to_i < 400
}
rescue StandardError => e
{ success: false, error: "Network error: #{e.message}" }
end
end
end
app.get '/auth' do
logger.info('GET /auth route handler entered (from AuthenticationRoutes)')
auth_manager = Legate::Auth::Manager.instance
schemes_count = auth_manager.schemes&.size || 0
credentials_count = auth_manager.credentials&.size || 0
mappings_count = auth_manager.url_mappings&.size || 0
instance_variable_set(:@auth_manager_available, true)
instance_variable_set(:@schemes_count, schemes_count)
instance_variable_set(:@credentials_count, credentials_count)
instance_variable_set(:@mappings_count, mappings_count)
slim :auth
rescue StandardError => e
logger.error("Error in /auth route (from AuthenticationRoutes): #{e.class} - #{e.message}")
instance_variable_set(:@auth_manager_available, false)
instance_variable_set(:@error_message, e.message)
slim :auth
end
app.get '/auth/schemes' do
logger.info('GET /auth/schemes route handler entered (from AuthenticationRoutes)')
content_type :html
auth_manager = Legate::Auth::Manager.instance
schemes = auth_manager.schemes || {}
credentials = auth_manager.credentials || {}
url_mappings = auth_manager.url_mappings || []
schemes_data = schemes.map do |name, scheme|
compatible_credentials = credentials.select do |cred_name, credential|
auth_manager.send(:credential_compatible_with_scheme?, credential, scheme)
end
scheme_mappings = url_mappings.select { |mapping| mapping[:scheme_name] == name }
{
name: name,
scheme_type: scheme.scheme_type,
class_name: scheme.class.name.split('::').last,
description: get_scheme_description(scheme),
compatible_credentials_count: compatible_credentials.size,
url_mappings_count: scheme_mappings.size,
config_fields: get_scheme_config_fields(scheme.scheme_type),
has_config: !get_scheme_config_fields(scheme.scheme_type).empty?
}
end
instance_variable_set(:@schemes, schemes_data)
slim :auth_schemes
rescue StandardError => e
logger.error("Error in /auth/schemes route (from AuthenticationRoutes): #{e.class} - #{e.message}")
halt 500, "Error loading authentication schemes: #{e.message}"
end
app.get '/auth/schemes/:name' do
logger.info("GET /auth/schemes/#{params[:name]} route handler entered (from AuthenticationRoutes)")
content_type :html
scheme_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
scheme = auth_manager.get_scheme(scheme_name)
halt 404, "Scheme not found: #{Rack::Utils.escape_html(params[:name])}" unless scheme
credentials = auth_manager.credentials || {}
url_mappings = auth_manager.url_mappings || []
compatible_credentials = credentials.select do |cred_name, credential|
auth_manager.send(:credential_compatible_with_scheme?, credential, scheme)
end.map do |cred_name, credential|
{
name: cred_name,
auth_type: credential.auth_type,
description: get_credential_description(credential),
masked_info: get_masked_credential_info(credential)
}
end
scheme_mappings = url_mappings.select { |mapping| mapping[:scheme_name] == scheme_name }
.map.with_index do |mapping, index|
{
id: index,
pattern: mapping[:pattern].is_a?(Regexp) ? mapping[:pattern].source : mapping[:pattern].to_s,
pattern_type: mapping[:pattern].is_a?(Regexp) ? 'regex' : 'string',
credential_name: mapping[:credential_name]
}
end
scheme_data = {
name: scheme_name,
scheme_type: scheme.scheme_type,
class_name: scheme.class.name.split('::').last,
description: get_scheme_description(scheme),
config_fields: get_scheme_config_fields(scheme.scheme_type),
current_config: get_scheme_current_config(scheme),
compatible_credential_types: get_compatible_credential_types(scheme.scheme_type),
compatible_credentials: compatible_credentials,
url_mappings: scheme_mappings
}
instance_variable_set(:@scheme, scheme_data)
slim :auth_scheme_detail
rescue StandardError => e
logger.error("Error in /auth/schemes/#{params[:name]} route (from AuthenticationRoutes): #{e.class} - #{e.message}")
halt 500, "Error loading scheme details: #{e.message}"
end
app.post '/auth/schemes' do
logger.info('POST /auth/schemes route handler entered (from AuthenticationRoutes)')
content_type :json
scheme_type = params[:scheme_type]&.to_sym
scheme_name = params[:scheme_name]&.to_sym
halt 400, { error: 'Scheme type is required' }.to_json unless scheme_type
halt 400, { error: 'Scheme name is required' }.to_json unless scheme_name
auth_manager = Legate::Auth::Manager.instance
halt 400, { error: "Scheme with name '#{scheme_name}' already exists" }.to_json if auth_manager.get_scheme(scheme_name)
begin
scheme = case scheme_type
when :api_key
Legate::Auth::Schemes::ApiKey.new
when :http_bearer
Legate::Auth::Schemes::HTTPBearer.new
when :oauth2
Legate::Auth::Schemes::OAuth2.new(
authorization_url: params[:authorization_url],
token_url: params[:token_url],
scopes: params[:scopes]&.split(/\s+/),
use_pkce: params[:use_pkce] == 'true',
revocation_url: params[:revocation_url]
)
when :oidc, :openid_connect
Legate::Auth::Schemes::OpenIDConnect.new(
authorization_url: params[:authorization_url],
token_url: params[:token_url],
userinfo_url: params[:userinfo_url],
scopes: params[:scopes]&.split(/\s+/),
use_pkce: params[:use_pkce] == 'true'
)
when :service_account
Legate::Auth::Schemes::ServiceAccount.new(
token_url: params[:token_url],
scopes: params[:scopes]&.split(/\s+/)
)
when :google_service_account
Legate::Auth::Schemes::GoogleServiceAccount.new(
scopes: params[:scopes]&.split(/\s+/)
)
else
halt 400, { error: "Unsupported scheme type: #{scheme_type}" }.to_json
end
auth_manager.register_scheme(scheme, scheme_name)
logger.info("Successfully registered new scheme '#{scheme_name}' of type '#{scheme_type}'")
{ success: true, message: "Scheme '#{scheme_name}' registered successfully" }.to_json
rescue StandardError => e
logger.error("Error registering scheme '#{scheme_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to register scheme: #{e.message}" }.to_json
end
end
app.put '/auth/schemes/:name' do
logger.info("PUT /auth/schemes/#{params[:name]} route handler entered (from AuthenticationRoutes)")
content_type :json
scheme_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
existing_scheme = auth_manager.get_scheme(scheme_name)
halt 404, { error: "Scheme not found: #{params[:name]}" }.to_json unless existing_scheme
begin
scheme_type = existing_scheme.scheme_type
updated_scheme = case scheme_type
when :oauth2
Legate::Auth::Schemes::OAuth2.new(
authorization_url: params[:authorization_url],
token_url: params[:token_url],
scopes: params[:scopes]&.split(/\s+/),
use_pkce: params[:use_pkce] == 'true',
revocation_url: params[:revocation_url]
)
when :oidc, :openid_connect
Legate::Auth::Schemes::OpenIDConnect.new(
authorization_url: params[:authorization_url],
token_url: params[:token_url],
userinfo_url: params[:userinfo_url],
scopes: params[:scopes]&.split(/\s+/),
use_pkce: params[:use_pkce] == 'true'
)
when :service_account
Legate::Auth::Schemes::ServiceAccount.new(
token_url: params[:token_url],
scopes: params[:scopes]&.split(/\s+/)
)
when :google_service_account
Legate::Auth::Schemes::GoogleServiceAccount.new(
scopes: params[:scopes]&.split(/\s+/)
)
else
halt 400, { error: "Scheme type '#{scheme_type}' does not support configuration updates" }.to_json
end
auth_manager.register_scheme(updated_scheme, scheme_name)
logger.info("Successfully updated scheme '#{scheme_name}' configuration")
{ success: true, message: "Scheme '#{scheme_name}' updated successfully" }.to_json
rescue StandardError => e
logger.error("Error updating scheme '#{scheme_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to update scheme: #{e.message}" }.to_json
end
end
app.delete '/auth/schemes/:name' do
logger.info("DELETE /auth/schemes/#{params[:name]} route handler entered (from AuthenticationRoutes)")
content_type :json
scheme_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
scheme = auth_manager.get_scheme(scheme_name)
halt 404, { error: "Scheme not found: #{params[:name]}" }.to_json unless scheme
url_mappings = auth_manager.url_mappings || []
dependent_mappings = url_mappings.select { |mapping| mapping[:scheme_name] == scheme_name }
if dependent_mappings.any?
mapping_patterns = dependent_mappings.map { |m| m[:pattern] }.join(', ')
halt 400, {
error: "Cannot delete scheme '#{scheme_name}' - it is used by URL mappings: #{mapping_patterns}"
}.to_json
end
begin
auth_manager.unregister_scheme(scheme_name)
logger.info("Successfully deleted scheme '#{scheme_name}'")
{ success: true, message: "Scheme '#{scheme_name}' deleted successfully" }.to_json
rescue StandardError => e
logger.error("Error deleting scheme '#{scheme_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to delete scheme: #{e.message}" }.to_json
end
end
app.get '/auth/credentials' do
logger.info('GET /auth/credentials route handler entered (from AuthenticationRoutes)')
content_type :html
auth_manager = Legate::Auth::Manager.instance
credentials = auth_manager.credentials || {}
url_mappings = auth_manager.url_mappings || []
credentials_data = credentials.map do |name, credential|
credential_mappings = url_mappings.select { |mapping| mapping[:credential_name] == name }
compatible_schemes = get_compatible_schemes_for_credential(credential.auth_type)
{
name: name,
auth_type: credential.auth_type,
description: get_credential_description(credential),
masked_info: get_masked_credential_info(credential),
url_mappings_count: credential_mappings.size,
compatible_schemes_count: compatible_schemes.size,
config_fields: get_credential_config_fields(credential.auth_type)
}
end
instance_variable_set(:@credentials, credentials_data)
slim :auth_credentials
rescue StandardError => e
logger.error("Error in /auth/credentials route (from AuthenticationRoutes): #{e.class} - #{e.message}")
halt 500, "Error loading authentication credentials: #{e.message}"
end
app.get '/auth/credentials/:name' do
logger.info("GET /auth/credentials/#{params[:name]} route handler entered (from AuthenticationRoutes)")
content_type :html
credential_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
credential = auth_manager.get_credential(credential_name)
halt 404, "Credential not found: #{Rack::Utils.escape_html(params[:name])}" unless credential
url_mappings = auth_manager.url_mappings || []
credential_mappings = url_mappings.select { |mapping| mapping[:credential_name] == credential_name }
.map.with_index do |mapping, index|
{
id: index,
pattern: mapping[:pattern].is_a?(Regexp) ? mapping[:pattern].source : mapping[:pattern].to_s,
pattern_type: mapping[:pattern].is_a?(Regexp) ? 'regex' : 'string',
scheme_name: mapping[:scheme_name]
}
end
compatible_schemes = get_compatible_schemes_for_credential(credential.auth_type)
credential_data = {
name: credential_name,
auth_type: credential.auth_type,
description: get_credential_description(credential),
config_fields: get_credential_config_fields(credential.auth_type),
current_config: get_credential_current_config(credential),
compatible_schemes: compatible_schemes,
url_mappings: credential_mappings
}
instance_variable_set(:@credential, credential_data)
slim :auth_credential_detail
rescue StandardError => e
logger.error("Error in /auth/credentials/#{params[:name]} route (from AuthenticationRoutes): #{e.class} - #{e.message}")
halt 500, "Error loading credential details: #{e.message}"
end
app.post '/auth/credentials' do
logger.info('POST /auth/credentials route handler entered (from AuthenticationRoutes)')
content_type :json
auth_type = params[:auth_type]&.to_sym
credential_name = params[:credential_name]&.to_sym
halt 400, { error: 'Credential type is required' }.to_json unless auth_type
halt 400, { error: 'Credential name is required' }.to_json unless credential_name
auth_manager = Legate::Auth::Manager.instance
halt 400, { error: "Credential with name '#{credential_name}' already exists" }.to_json if auth_manager.get_credential(credential_name)
begin
credential_attrs = { auth_type: auth_type }
case auth_type
when :api_key
credential_attrs[:api_key] = params[:api_key]
credential_attrs[:location] = params[:location] if params[:location] && !params[:location].empty?
credential_attrs[:name] = params[:name] if params[:name] && !params[:name].empty?
when :http_bearer
credential_attrs[:bearer_token] = params[:bearer_token]
when :oauth2, :oidc
credential_attrs[:client_id] = params[:client_id]
credential_attrs[:client_secret] = params[:client_secret]
credential_attrs[:redirect_uri] = params[:redirect_uri] if params[:redirect_uri] && !params[:redirect_uri].empty?
credential_attrs[:scopes] = params[:scopes] if params[:scopes] && !params[:scopes].empty?
when :service_account
credential_attrs[:client_email] = params[:client_email]
credential_attrs[:private_key] = params[:private_key]
credential_attrs[:project_id] = params[:project_id] if params[:project_id] && !params[:project_id].empty?
when :google_service_account
credential_attrs[:service_account_key] = params[:service_account_key]
when :basic
credential_attrs[:username] = params[:username]
credential_attrs[:password] = params[:password]
else
halt 400, { error: "Unsupported credential type: #{auth_type}" }.to_json
end
credential = Legate::Auth::Credential.new(**credential_attrs)
auth_manager.register_credential(credential, credential_name)
logger.info("Successfully registered new credential '#{credential_name}' of type '#{auth_type}'")
{ success: true, message: "Credential '#{credential_name}' created successfully" }.to_json
rescue Legate::Auth::CredentialError => e
logger.error("Credential validation error for '#{credential_name}': #{e.message}")
halt 400, { error: "Invalid credential: #{e.message}" }.to_json
rescue StandardError => e
logger.error("Error creating credential '#{credential_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to create credential: #{e.message}" }.to_json
end
end
app.put '/auth/credentials/:name' do
logger.info("PUT /auth/credentials/#{params[:name]} route handler entered (from AuthenticationRoutes)")
content_type :json
credential_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
existing_credential = auth_manager.get_credential(credential_name)
halt 404, { error: "Credential not found: #{params[:name]}" }.to_json unless existing_credential
begin
auth_type = existing_credential.auth_type
credential_attrs = { auth_type: auth_type }
case auth_type
when :api_key
credential_attrs[:api_key] = params[:api_key]
credential_attrs[:location] = params[:location] if params[:location] && !params[:location].empty?
credential_attrs[:name] = params[:name] if params[:name] && !params[:name].empty?
when :http_bearer
credential_attrs[:bearer_token] = params[:bearer_token]
when :oauth2, :oidc
credential_attrs[:client_id] = params[:client_id]
credential_attrs[:client_secret] = params[:client_secret]
credential_attrs[:redirect_uri] = params[:redirect_uri] if params[:redirect_uri] && !params[:redirect_uri].empty?
credential_attrs[:scopes] = params[:scopes] if params[:scopes] && !params[:scopes].empty?
when :service_account
credential_attrs[:client_email] = params[:client_email]
credential_attrs[:private_key] = params[:private_key]
credential_attrs[:project_id] = params[:project_id] if params[:project_id] && !params[:project_id].empty?
when :google_service_account
credential_attrs[:service_account_key] = params[:service_account_key]
when :basic
credential_attrs[:username] = params[:username]
credential_attrs[:password] = params[:password]
else
halt 400, { error: "Credential type '#{auth_type}' does not support updates" }.to_json
end
updated_credential = Legate::Auth::Credential.new(**credential_attrs)
auth_manager.register_credential(updated_credential, credential_name)
logger.info("Successfully updated credential '#{credential_name}'")
{ success: true, message: "Credential '#{credential_name}' updated successfully" }.to_json
rescue Legate::Auth::CredentialError => e
logger.error("Credential validation error for '#{credential_name}': #{e.message}")
halt 400, { error: "Invalid credential: #{e.message}" }.to_json
rescue StandardError => e
logger.error("Error updating credential '#{credential_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to update credential: #{e.message}" }.to_json
end
end
app.delete '/auth/credentials/:name' do
logger.info("DELETE /auth/credentials/#{params[:name]} route handler entered (from AuthenticationRoutes)")
content_type :json
credential_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
credential = auth_manager.get_credential(credential_name)
halt 404, { error: "Credential not found: #{params[:name]}" }.to_json unless credential
url_mappings = auth_manager.url_mappings || []
dependent_mappings = url_mappings.select { |mapping| mapping[:credential_name] == credential_name }
if dependent_mappings.any?
mapping_patterns = dependent_mappings.map { |m| m[:pattern] }.join(', ')
halt 400, {
error: "Cannot delete credential '#{credential_name}' - it is used by URL mappings: #{mapping_patterns}"
}.to_json
end
begin
auth_manager.unregister_credential(credential_name)
logger.info("Successfully deleted credential '#{credential_name}'")
{ success: true, message: "Credential '#{credential_name}' deleted successfully" }.to_json
rescue StandardError => e
logger.error("Error deleting credential '#{credential_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to delete credential: #{e.message}" }.to_json
end
end
app.post '/auth/credentials/:name/test' do
logger.info("POST /auth/credentials/#{params[:name]}/test route handler entered (from AuthenticationRoutes)")
content_type :json
credential_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
credential = auth_manager.get_credential(credential_name)
halt 404, { error: "Credential not found: #{params[:name]}" }.to_json unless credential
begin
test_result = { success: true, tests: [] }
begin
credential.to_h(resolve_env: true)
test_result[:tests] << {
name: 'Environment Variable Resolution',
status: 'passed',
message: 'All environment variables resolved successfully'
}
rescue Legate::Auth::EnvironmentVariableNotFoundError => e
test_result[:success] = false
test_result[:tests] << {
name: 'Environment Variable Resolution',
status: 'failed',
message: "Environment variable not found: #{e.message}"
}
end
begin
Legate::Auth::Credential.new(**credential.to_h(resolve_env: false))
test_result[:tests] << {
name: 'Required Fields Validation',
status: 'passed',
message: 'All required fields are present'
}
rescue Legate::Auth::CredentialError => e
test_result[:success] = false
test_result[:tests] << {
name: 'Required Fields Validation',
status: 'failed',
message: e.message
}
end
case credential.auth_type
when :google_service_account
begin
key_data = credential[:service_account_key]
if key_data
JSON.parse(key_data)
test_result[:tests] << {
name: 'Service Account Key Format',
status: 'passed',
message: 'Service account key is valid JSON'
}
end
rescue JSON::ParserError
test_result[:success] = false
test_result[:tests] << {
name: 'Service Account Key Format',
status: 'failed',
message: 'Service account key is not valid JSON'
}
end
when :service_account
begin
private_key = credential[:private_key]
if private_key && !private_key.include?('BEGIN PRIVATE KEY')
test_result[:success] = false
test_result[:tests] << {
name: 'Private Key Format',
status: 'failed',
message: 'Private key does not appear to be in PEM format'
}
else
test_result[:tests] << {
name: 'Private Key Format',
status: 'passed',
message: 'Private key appears to be in correct PEM format'
}
end
rescue StandardError => e
test_result[:success] = false
test_result[:tests] << {
name: 'Private Key Format',
status: 'failed',
message: "Private key validation error: #{e.message}"
}
end
end
logger.info("Credential test completed for '#{credential_name}': #{test_result[:success] ? 'PASSED' : 'FAILED'}")
test_result.to_json
rescue StandardError => e
logger.error("Error testing credential '#{credential_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to test credential: #{e.message}" }.to_json
end
end
app.get '/auth/mappings' do
logger.info('GET /auth/mappings route handler entered (from AuthenticationRoutes)')
content_type :html
auth_manager = Legate::Auth::Manager.instance
mappings = auth_manager.url_mappings || []
mappings_data = mappings.map.with_index do |mapping, index|
{
id: index,
pattern: mapping[:pattern].is_a?(Regexp) ? mapping[:pattern].source : mapping[:pattern].to_s,
pattern_type: mapping[:pattern].is_a?(Regexp) ? 'regex' : 'string',
scheme_name: mapping[:scheme_name],
credential_name: mapping[:credential_name]
}
end
instance_variable_set(:@mappings, mappings_data)
slim :auth_mappings
rescue StandardError => e
logger.error("Error in /auth/mappings route (from AuthenticationRoutes): #{e.class} - #{e.message}")
halt 500, "Error loading URL mappings: #{e.message}"
end
app.get '/auth/debug' do
logger.info('GET /auth/debug route handler entered (from AuthenticationRoutes)')
content_type :html
auth_manager = Legate::Auth::Manager.instance
debug_info = {
manager_class: auth_manager.class.name,
schemes_registered: auth_manager.schemes&.keys || [],
credentials_registered: auth_manager.credentials&.keys || [],
url_mappings_count: auth_manager.url_mappings&.size || 0,
manager_instance_id: auth_manager.object_id
}
instance_variable_set(:@debug_info, debug_info)
slim :auth_debug
rescue StandardError => e
logger.error("Error in /auth/debug route (from AuthenticationRoutes): #{e.class} - #{e.message}")
halt 500, "Error gathering debug information: #{e.message}"
end
app.get '/auth/test' do
logger.info('GET /auth/test route handler entered (from AuthenticationRoutes)')
content_type :html
auth_manager = Legate::Auth::Manager.instance
schemes = auth_manager.schemes || {}
credentials = auth_manager.credentials || {}
mappings = auth_manager.url_mappings || []
schemes_data = schemes.map do |name, scheme|
{
name: name,
scheme_type: scheme.scheme_type,
description: get_scheme_description(scheme)
}
end
credentials_data = credentials.map do |name, credential|
{
name: name,
auth_type: credential.auth_type,
description: get_credential_description(credential)
}
end
instance_variable_set(:@schemes, schemes_data)
instance_variable_set(:@credentials, credentials_data)
instance_variable_set(:@mappings_count, mappings.size)
slim :auth_test
rescue StandardError => e
logger.error("Error in /auth/test route (from AuthenticationRoutes): #{e.class} - #{e.message}")
halt 500, "Error loading testing dashboard: #{e.message}"
end
app.post '/auth/test/credential/:name' do
logger.info("POST /auth/test/credential/#{params[:name]} route handler entered (from AuthenticationRoutes)")
content_type :json
credential_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
credential = auth_manager.get_credential(credential_name)
halt 404, { error: "Credential not found: #{params[:name]}" }.to_json unless credential
begin
test_options = {}
test_options[:test_url] = params[:test_url] if params[:test_url] && !params[:test_url].empty?
test_results = test_credential_functionality(credential, test_options)
test_results[:credential_name] = credential_name.to_s
logger.info("Credential test completed for '#{credential_name}': #{test_results[:success] ? 'PASSED' : 'FAILED'}")
test_results.to_json
rescue StandardError => e
logger.error("Error testing credential '#{credential_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to test credential: #{e.message}" }.to_json
end
end
app.post '/auth/test/scheme/:name' do
logger.info("POST /auth/test/scheme/#{params[:name]} route handler entered (from AuthenticationRoutes)")
content_type :json
scheme_name = params[:name].to_sym
auth_manager = Legate::Auth::Manager.instance
scheme = auth_manager.get_scheme(scheme_name)
halt 404, { error: "Scheme not found: #{params[:name]}" }.to_json unless scheme
begin
test_results = { success: true, tests: [], scheme_name: scheme_name.to_s }
case scheme.scheme_type
when :oauth2, :oidc, :openid_connect
if scheme.respond_to?(:authorization_url) && scheme.authorization_url
test_results[:tests] << {
name: 'Authorization URL',
status: 'passed',
message: "Valid authorization URL: #{scheme.authorization_url}"
}
else
test_results[:success] = false
test_results[:tests] << {
name: 'Authorization URL',
status: 'failed',
message: 'Authorization URL is missing or invalid'
}
end
if scheme.respond_to?(:token_url) && scheme.token_url
test_results[:tests] << {
name: 'Token URL',
status: 'passed',
message: "Valid token URL: #{scheme.token_url}"
}
else
test_results[:success] = false
test_results[:tests] << {
name: 'Token URL',
status: 'failed',
message: 'Token URL is missing or invalid'
}
end
when :service_account, :google_service_account
test_results[:tests] << if scheme.respond_to?(:token_url) && scheme.token_url
{
name: 'Token URL',
status: 'passed',
message: "Valid token URL: #{scheme.token_url}"
}
else
{
name: 'Token URL',
status: 'skipped',
message: 'No token URL configured (using default)'
}
end
else
test_results[:tests] << {
name: 'Scheme Configuration',
status: 'passed',
message: "Scheme type #{scheme.scheme_type} requires no additional configuration"
}
end
credentials = auth_manager.credentials || {}
compatible_credentials = credentials.select do |cred_name, credential|
mapping_scheme_credential_compatible?(scheme, credential)
end
test_results[:tests] << if compatible_credentials.any?
{
name: 'Compatible Credentials',
status: 'passed',
message: "Found #{compatible_credentials.size} compatible credential(s)"
}
else
{
name: 'Compatible Credentials',
status: 'warning',
message: 'No compatible credentials found'
}
end
logger.info("Scheme test completed for '#{scheme_name}': #{test_results[:success] ? 'PASSED' : 'FAILED'}")
test_results.to_json
rescue StandardError => e
logger.error("Error testing scheme '#{scheme_name}': #{e.class} - #{e.message}")
halt 500, { error: "Failed to test scheme: #{e.message}" }.to_json
end
end
app.post '/auth/test/api' do
logger.info('POST /auth/test/api route handler entered (from AuthenticationRoutes)')
content_type :json
url = params[:url]
scheme_name = params[:scheme_name]
credential_name = params[:credential_name]
halt 400, { error: 'URL is required' }.to_json unless url && !url.empty?
halt 400, { error: 'Scheme name is required' }.to_json unless scheme_name && !scheme_name.empty?
halt 400, { error: 'Credential name is required' }.to_json unless credential_name && !credential_name.empty?
begin
result = test_authenticated_api_call(url, scheme_name, credential_name)
logger.info("API test completed for URL '#{url}' with scheme '#{scheme_name}' and credential '#{credential_name}': #{result[:success] ? 'SUCCESS' : 'FAILED'}")
result.to_json
rescue StandardError => e
logger.error("Error testing API call: #{e.class} - #{e.message}")
halt 500, { error: "Failed to test API call: #{e.message}" }.to_json
end
end
app.post '/auth/test/flow' do
logger.info('POST /auth/test/flow route handler entered (from AuthenticationRoutes)')
content_type :json
scheme_name = params[:scheme_name]
credential_name = params[:credential_name]
halt 400, { error: 'Scheme name is required' }.to_json unless scheme_name && !scheme_name.empty?
halt 400, { error: 'Credential name is required' }.to_json unless credential_name && !credential_name.empty?
begin
auth_manager = Legate::Auth::Manager.instance
scheme = auth_manager.get_scheme(scheme_name.to_sym)
credential = auth_manager.get_credential(credential_name.to_sym)
halt 404, { error: 'Scheme not found' }.to_json unless scheme
halt 404, { error: 'Credential not found' }.to_json unless credential
halt 400, { error: 'Scheme and credential are not compatible' }.to_json unless mapping_scheme_credential_compatible?(scheme, credential)
flow_results = { success: true, steps: [], scheme_type: scheme.scheme_type }
case scheme.scheme_type
when :api_key
flow_results[:steps] << {
step: 'API Key Authentication',
status: 'passed',
message: 'API key authentication is ready for use'
}
when :http_bearer
flow_results[:steps] << {
step: 'Bearer Token Authentication',
status: 'passed',
message: 'Bearer token authentication is ready for use'
}
when :oauth2, :oidc, :openid_connect
flow_results[:steps] << {
step: 'OAuth2 Flow Simulation',
status: 'simulated',
message: 'OAuth2 flow would redirect to authorization URL and exchange code for token'
}
when :service_account, :google_service_account
flow_results[:steps] << {
step: 'Service Account Flow Simulation',
status: 'simulated',
message: 'Service account would generate JWT and exchange for access token'
}
else
flow_results[:success] = false
flow_results[:steps] << {
step: 'Unknown Flow',
status: 'failed',
message: "No flow simulation available for scheme type: #{scheme.scheme_type}"
}
end
logger.info("Flow test completed for scheme '#{scheme_name}' and credential '#{credential_name}': #{flow_results[:success] ? 'SUCCESS' : 'FAILED'}")
flow_results.to_json
rescue StandardError => e
logger.error("Error testing authentication flow: #{e.class} - #{e.message}")
halt 500, { error: "Failed to test authentication flow: #{e.message}" }.to_json
end
end
app.get '/auth/mappings/new' do
auth_manager = Legate::Auth::Manager.instance
schemes = auth_manager.schemes || {}
credentials = auth_manager.credentials || {}
mapping_fields = get_mapping_config_fields
instance_variable_set(:@schemes, schemes)
instance_variable_set(:@credentials, credentials)
instance_variable_set(:@mapping_fields, mapping_fields)
slim :auth_mapping_new
end
app.get '/auth/mappings/:id' do
auth_manager = Legate::Auth::Manager.instance
mappings = auth_manager.url_mappings || []
id = params[:id].to_i
mapping = mappings[id] or halt 404, 'Mapping not found'
schemes = auth_manager.schemes || {}
credentials = auth_manager.credentials || {}
mapping_fields = get_mapping_config_fields
instance_variable_set(:@mapping, mapping)
instance_variable_set(:@schemes, schemes)
instance_variable_set(:@credentials, credentials)
instance_variable_set(:@mapping_fields, mapping_fields)
slim :auth_mapping_detail
end
app.post '/auth/mappings' do
content_type :json
auth_manager = Legate::Auth::Manager.instance
mappings = auth_manager.url_mappings || []
pattern = params[:pattern]
pattern_type = params[:pattern_type]
scheme_name = params[:scheme_name]&.to_sym
credential_name = params[:credential_name]&.to_sym
priority = params[:priority]&.to_i || 1
active = %w[on true].include?(params[:active])
halt 400, { error: 'Invalid pattern' }.to_json unless valid_mapping_pattern?(pattern, pattern_type)
scheme = auth_manager.get_scheme(scheme_name)
credential = auth_manager.get_credential(credential_name)
halt 400, { error: 'Scheme and credential are not compatible' }.to_json unless mapping_scheme_credential_compatible?(scheme, credential)
mapping = {
pattern: pattern_type == 'regex' ? Regexp.new(pattern) : pattern,
pattern_type: pattern_type,
scheme_name: scheme_name,
credential_name: credential_name,
priority: priority,
active: active
}
mappings << mapping
auth_manager.replace_url_mappings(mappings)
{ success: true, message: 'Mapping created', id: mappings.size - 1 }.to_json
end
app.put '/auth/mappings/:id' do
content_type :json
auth_manager = Legate::Auth::Manager.instance
mappings = auth_manager.url_mappings || []
id = params[:id].to_i
mapping = mappings[id] or halt 404, { error: 'Mapping not found' }.to_json
pattern = params[:pattern]
pattern_type = params[:pattern_type]
scheme_name = params[:scheme_name]&.to_sym
credential_name = params[:credential_name]&.to_sym
priority = params[:priority]&.to_i || 1
active = %w[on true].include?(params[:active])
halt 400, { error: 'Invalid pattern' }.to_json unless valid_mapping_pattern?(pattern, pattern_type)
scheme = auth_manager.get_scheme(scheme_name)
credential = auth_manager.get_credential(credential_name)
halt 400, { error: 'Scheme and credential are not compatible' }.to_json unless mapping_scheme_credential_compatible?(scheme, credential)
mapping[:pattern] = pattern_type == 'regex' ? Regexp.new(pattern) : pattern
mapping[:pattern_type] = pattern_type
mapping[:scheme_name] = scheme_name
mapping[:credential_name] = credential_name
mapping[:priority] = priority
mapping[:active] = active
auth_manager.replace_url_mappings(mappings)
{ success: true, message: 'Mapping updated' }.to_json
end
app.delete '/auth/mappings/:id' do
content_type :json
auth_manager = Legate::Auth::Manager.instance
mappings = auth_manager.url_mappings || []
id = params[:id].to_i
mapping = mappings[id] or halt 404, { error: 'Mapping not found' }.to_json
mappings.delete_at(id)
auth_manager.replace_url_mappings(mappings)
{ success: true, message: 'Mapping deleted' }.to_json
end
app.post '/auth/mappings/test' do
content_type :json
pattern = params[:pattern]
pattern_type = params[:pattern_type]
url = params[:url]
halt 400, { error: 'Invalid pattern' }.to_json unless valid_mapping_pattern?(pattern, pattern_type)
begin
matched =
if pattern_type == 'regex'
!!(url =~ Regexp.new(pattern))
elsif pattern.include?('*')
regex = Regexp.new('^' + Regexp.escape(pattern).gsub('\\*', '.*') + '$')
!!(url =~ regex)
else
url == pattern
end
{ success: true, matched: matched }.to_json
rescue StandardError => e
halt 400, { error: "Pattern test error: #{e.message}" }.to_json
end
end
end
|